import {
  MAGIC_COMMENT_STATIC,
  REGEX_JSX_FILE,
  REGEX_TS_FILE
} from "./chunk-2CWCRQZP.mjs";

// src/ast.ts
import path from "path";
import { babelParse as _babelParse, walkIdentifiers } from "@vue/compiler-sfc";
function getLang(filename) {
  return path.extname(filename).replace(/^\./, "");
}
function isTs(lang) {
  return lang && REGEX_TS_FILE.test(lang);
}
function babelParse(code, lang, options = {}) {
  const plugins = [];
  if (lang) {
    if (isTs(lang))
      plugins.push("typescript");
    if (REGEX_JSX_FILE.test(lang))
      plugins.push("jsx");
  }
  const { program } = _babelParse(code, {
    sourceType: "module",
    plugins,
    ...options
  });
  return program;
}
function isCallOf(node, test) {
  return !!(node && node.type === "CallExpression" && node.callee.type === "Identifier" && (typeof test === "string" ? node.callee.name === test : test(node.callee.name)));
}
function checkInvalidScopeReference(node, method, setupBindings) {
  if (!node)
    return;
  walkIdentifiers(node, (id) => {
    if (setupBindings.includes(id.name))
      throw new SyntaxError(
        `\`${method}()\` in <script setup> cannot reference locally declared variables (${id.name}) because it will be hoisted outside of the setup() function.`
      );
  });
}
function isStaticExpression(node) {
  var _a;
  if ((_a = node.leadingComments) == null ? void 0 : _a.some(
    (comment) => comment.value.trim() === MAGIC_COMMENT_STATIC
  ))
    return true;
  switch (node.type) {
    case "UnaryExpression":
      return isStaticExpression(node.argument);
    case "LogicalExpression":
    case "BinaryExpression":
      return isStaticExpression(node.left) && isStaticExpression(node.right);
    case "ConditionalExpression":
      return isStaticExpression(node.test) && isStaticExpression(node.consequent) && isStaticExpression(node.alternate);
    case "SequenceExpression":
    case "TemplateLiteral":
      return node.expressions.every((expr) => isStaticExpression(expr));
    case "ParenthesizedExpression":
    case "TSNonNullExpression":
    case "TSAsExpression":
    case "TSTypeAssertion":
      return isStaticExpression(node.expression);
  }
  if (isLiteralType(node))
    return true;
  return false;
}
function isLiteralType(node) {
  return node.type.endsWith("Literal");
}
function getStaticKey(node, computed = false, raw = true) {
  switch (node.type) {
    case "StringLiteral":
    case "NumericLiteral":
      return raw ? node.extra.raw : node.value;
    case "Identifier":
      if (!computed)
        return raw ? `'${node.name}'` : node.name;
    default:
      throw new SyntaxError(`Unexpected node type: ${node.type}`);
  }
}

export {
  getLang,
  isTs,
  babelParse,
  isCallOf,
  checkInvalidScopeReference,
  isStaticExpression,
  isLiteralType,
  getStaticKey
};
